import java.sql.*; 
import java.io.*; 

class TableLister { 
  public static void main (String args []) 
      throws SQLException, IOException { 

    //   Open Oracle Connection.
    try {
      Class.forName ("oracle.jdbc.driver.OracleDriver");
    } catch (ClassNotFoundException e) {
        System.out.println ("Could not load the driver"); 
    }

    String user, pass;
    user = readEntry("userid  : ");
    pass = readEntry("password: ");
    Connection conn =  DriverManager.getConnection
                            ("jdbc:oracle:oci8:@addb", user, pass);

    System.out.println("Welcome to the Oracle Table Lister.\n");
    Statement stmt = conn.createStatement(); 
    do {
      String command = readCommand();

      //   exit.
      if (command.equals("exit")) 
        break;

      if (command.equals("help") || command.equals("?")) {
          System.out.println("Type the table name to list its contents or use the following commands:");
          System.out.println("    ?   :   this help menu.");
          System.out.println("    help:   this help menu.");
          System.out.println("    all :   list all table names.");
          continue;
      }

      ResultSet rset = null;
      //   list all table names.
      if (command.equals("all")) {
         try {          
             rset = stmt.executeQuery("select * from tab");
         } catch (SQLException e) {
             System.out.println("Table names cannot be listed: " + e.getMessage());
             continue;
         } 
       
         //  list tables using the table tab.
         System.out.println("All tables in this database:");
         while (rset.next()) {
              System.out.println("  " + rset.getString(1));
         }

         //  skip to the next command.
         continue;
      }

      //  List table result.
      try {
          rset = stmt.executeQuery("select * from " + command);
      } catch (SQLException e) {
          System.out.print("Table " + command + "cannot be found.");
          continue;
      }
 
      ResultSetMetaData rsetmd = rset.getMetaData();
      int nCols = rsetmd.getColumnCount();
      int totalCols = nCols;

      for (int i = 1; i <= nCols; i++) {
        System.out.print(rsetmd.getColumnName(i));
        int colSize = rsetmd.getColumnDisplaySize(i);
        totalCols += colSize;

        for (int k=0; k < colSize-rsetmd.getColumnName(i).length(); k++)
          System.out.print(" ");
        System.out.print(" ");
      }
      System.out.println("");

      for (int i=0; i<totalCols; i++) 
          System.out.print("-");
      System.out.println("");

      while (rset.next ()) { 
        for (int i = 1; i <= nCols; i++) {
          String val = rset.getString(i);
          int colSize = rsetmd.getColumnDisplaySize(i);
          int printedSize = 0;

          if (rset.wasNull()) {
             System.out.print("null");
             printedSize = 4;
          }
          else {
              System.out.print(rset.getString(i));
              printedSize = rset.getString(i).length();
          }
          for (int k=0; k <= colSize-printedSize; k++)
              System.out.print(" ");
          
        }
        System.out.println("");
      } 
    } while (true);  
    stmt.close();
    conn.close();
    System.out.println("Thank you for using the Table Lister\n");
  } 

  //readEntry function -- to read input string
  static String readCommand() {
      return readEntry("Table Lister>");
  }

  //readEntry function -- to read input string
  static String readEntry(String prompt) {
     try {
       StringBuffer buffer = new StringBuffer();
       System.out.print(prompt);
       System.out.flush();
       int c = System.in.read();
       while(c != '\n' && c != -1) {
         buffer.append((char)c);
         c = System.in.read();
       }
       return buffer.toString().trim();
     } catch (IOException e) {
       return "";
       }
   }

} 
